import '../../styles';
import * as React from 'react';
import { Controlled as CodeMirrorControlled } from 'react-codemirror2';
import { Subject } from 'rxjs';
import { share, takeUntil, filter, map } from 'rxjs/operators';
import { css, is, time, value as valueUtil, events } from '../../common';
if (is.browser) {
    require('codemirror/mode/spreadsheet/spreadsheet.js');
}
export class FormulaInput extends React.PureComponent {
    constructor() {
        super(...arguments);
        this.state = { isLoaded: false };
        this.unmounted$ = new Subject();
        this.state$ = new Subject();
        this.editorRef = (ref) => (this.editor = ref && ref.editor);
        this._events$ = new Subject();
        this.events$ = this._events$.pipe(takeUntil(this.unmounted$), share());
        this.modifierKeys = {
            alt: false,
            control: false,
            shift: false,
            meta: false,
        };
        this.fire = (e) => this._events$.next(e);
        this.focusHandler = (isFocused) => {
            return () => {
                if (isFocused) {
                    this.fire({ type: 'INPUT/formula/focus', payload: {} });
                }
                if (!isFocused) {
                    this.fire({ type: 'INPUT/formula/blur', payload: {} });
                }
            };
        };
        this.handleBeforeChange = (editor, data, value) => {
            const { multiline: isMultiLine = false, maxLength } = this.props;
            const change = data;
            const char = change.text[0];
            const modifierKeys = Object.assign({}, this.modifierKeys);
            if (char.includes('\t')) {
                let isCancelled = false;
                this.fire({
                    type: 'INPUT/formula/tab',
                    payload: {
                        get isCancelled() {
                            return isCancelled;
                        },
                        cancel() {
                            isCancelled = true;
                        },
                        modifierKeys,
                    },
                });
                if (isCancelled) {
                    return;
                }
            }
            if (!isMultiLine) {
                const wasCancelled = singleLineOnly(change);
                if (wasCancelled) {
                    return;
                }
            }
            if (isNewLine(change)) {
                let isCancelled = false;
                this.fire({
                    type: 'INPUT/formula/newLine',
                    payload: {
                        get isCancelled() {
                            return isCancelled;
                        },
                        cancel() {
                            isCancelled = true;
                        },
                        modifierKeys,
                    },
                });
                if (isCancelled) {
                    return;
                }
            }
            const from = this.props.value || '';
            const to = value;
            const isMax = maxLength === undefined ? null : to.length === maxLength;
            let isChangeCancelled = false;
            const changedPayload = { from, to, isMax, char, modifierKeys };
            const changingPayload = Object.assign({}, changedPayload, { get isCancelled() {
                    return isChangeCancelled;
                },
                cancel() {
                    isChangeCancelled = true;
                } });
            this.fire({ type: 'INPUT/formula/changing', payload: changingPayload });
            if (isChangeCancelled) {
                return;
            }
            if (maxLength !== undefined) {
                const clippedValue = formatValue(value, { maxLength, isMultiLine });
                if (clippedValue !== value) {
                    if (change.update && change.origin === '+input') {
                        change.update(undefined, undefined, [clippedValue]);
                        value = clippedValue;
                    }
                }
            }
            this.fire({ type: 'INPUT/formula/changed', payload: changedPayload });
        };
    }
    componentWillMount() {
        const { events$ } = this.props;
        this.state$.pipe(takeUntil(this.unmounted$)).subscribe(e => this.setState(e));
        if (events$) {
            this.events$.subscribe(e => events$.next(e));
        }
        this.events$
            .pipe(filter(e => e.type === 'INPUT/formula/tab'), filter(e => !valueUtil.defaultValue(this.props.allowTab, true)), map(e => e.payload))
            .subscribe(e => e.cancel());
        const keypress$ = events.keyPress$.pipe(takeUntil(this.unmounted$));
        const modifier$ = keypress$.pipe(filter(e => e.isModifier));
        modifier$
            .pipe(filter(e => e.isPressed), map(e => e.key.toLowerCase()))
            .subscribe(key => (this.modifierKeys[key] = true));
        modifier$
            .pipe(filter(e => !e.isPressed), map(e => e.key.toLowerCase()))
            .subscribe(key => (this.modifierKeys[key] = false));
    }
    componentDidMount() {
        time.delay(0, () => {
            this.setState({ isLoaded: true }, () => this.init());
        });
    }
    init() {
        const { focusOnLoad, selectOnLoad } = this.props;
        if (focusOnLoad) {
            this.focus();
        }
        if (selectOnLoad) {
            this.selectAll();
        }
    }
    componentWillUnmount() {
        this.unmounted$.next();
        this.unmounted$.complete();
    }
    get isFocused() {
        return this.editor ? this.editor.hasFocus() : false;
    }
    get height() {
        const { multiline: isMultiLine = false, height } = this.props;
        if (height !== undefined) {
            return height;
        }
        if (!isMultiLine && height === undefined) {
            return 21;
        }
        return;
    }
    focus() {
        if (this.editor) {
            this.editor.focus();
        }
    }
    redraw() {
        this.forceUpdate();
        if (this.editor) {
            this.editor.refresh();
        }
    }
    selectAll() {
        this.editor.execCommand('selectAll');
    }
    render() {
        const { multiline: isMultiLine = false, maxLength, mode = 'spreadsheet' } = this.props;
        const height = this.height;
        const value = formatValue(this.props.value, { maxLength, isMultiLine });
        const styles = {
            base: css({
                height,
                Scroll: isMultiLine,
                overflow: 'hidden',
                lineHeight: 1,
            }),
        };
        const el = this.state.isLoaded && (React.createElement(CodeMirrorControlled, { ref: this.editorRef, options: { mode }, value: value, onBeforeChange: this.handleBeforeChange, onFocus: this.focusHandler(true), onBlur: this.focusHandler(false) }));
        return (React.createElement("div", Object.assign({ className: 'FormulaInput' }, css(styles.base, this.props.style)), el));
    }
}
function singleLineOnly(change) {
    if (isNewLine(change)) {
        return change.cancel();
    }
    const pastedNewLine = change.origin === 'paste' && typeof change.text === 'object' && change.text.length > 1;
    if (pastedNewLine && change.update) {
        const newText = change.text.join(' ');
        return change.update(undefined, undefined, [newText]);
    }
    return null;
}
function isNewLine(change) {
    return (change.origin === '+input' && typeof change.text === 'object' && change.text.join('') === '');
}
function formatValue(value, options) {
    const { maxLength, isMultiLine = false } = options;
    value = value || '';
    if (!isMultiLine) {
        value = value.replace(/\n/g, '');
    }
    if (maxLength !== undefined && value.length > maxLength) {
        value = value.substr(0, maxLength);
    }
    return value;
}
